@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2006 - 2016, Paul Beckingham, Federico Hernandez.
+// Copyright 2016 - 2020, Thomas Lauf, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -20,7 +20,7 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -33,6 +33,10 @@
 #include <utf8.h>
 #include <sstream>
 #include <algorithm>
+#include <set>
+#include <Duration.h>
+#include <timew.h>
+#include "DatetimeParser.h"
 
 ////////////////////////////////////////////////////////////////////////////////
 A2::A2 (const std::string& raw, Lexer::Type lextype)
@@ -326,7 +330,7 @@ bool CLI::canonicalize (
 ////////////////////////////////////////////////////////////////////////////////
 std::string CLI::getBinary () const
 {
-  if (_args.size ())
+  if (! _args.empty ())
     return _args[0].attribute ("raw");
 
   return "";
@@ -365,7 +369,7 @@ std::string CLI::dump (const std::string& title) const
 
   out << '\n';
 
-  if (_args.size ())
+  if (! _args.empty ())
   {
     out << "  _args\n";
     for (auto& a : _args)
@@ -413,6 +417,9 @@ void CLI::identifyIds ()
           pig.getDigits (digits) &&
           pig.eos ())
       {
+        if (digits <= 0)
+          throw format ("'@{1}' is not a valid ID.", digits);
+
         a.tag ("ID");
         a.attribute ("value", digits);
       }
@@ -441,6 +448,22 @@ void CLI::canonicalizeNames ()
       alreadyFoundCmd = true;
     }
 
+    // 'timew <command> --help|-h' should be treated the same as 'timew help <command>'.
+    // Therefore, '--help|-h' on the command line should always become the command.
+    else if (alreadyFoundCmd && (raw == "--help" || raw == "-h"))
+    {
+      for (auto& b : _args) {
+        if (b.hasTag("CMD"))
+        {
+          b.unTag("CMD");
+          break;
+        }
+      }
+
+      a.tag ("CMD");
+      a.attribute("canonical", canonical);
+    }
+
     // Hints.
     else if (exactMatch ("hint", raw) ||
              canonicalize (canonical, "hint", raw))
@@ -500,6 +523,10 @@ void CLI::identifyFilter ()
       a.tag ("KEYWORD");
     }
 
+    else if (raw.rfind("dom.",0) == 0)
+    {
+      a.tag ("DOM");
+    }
     else
     {
       a.tag ("FILTER");
@@ -524,17 +551,285 @@ bool CLI::exactMatch (
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-std::vector<int> CLI::getIds() const
+std::set<int> CLI::getIds() const
 {
-  std::vector <int> ids;
+  std::set <int> ids;
 
   for (auto& arg : _args)
   {
     if (arg.hasTag ("ID"))
-      ids.push_back (strtol (arg.attribute ("value").c_str (), NULL, 10));
+      ids.insert (strtol (arg.attribute ("value").c_str (), NULL, 10));
   }
 
   return ids;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
+std::vector<std::string> CLI::getTags () const
+{
+  std::vector <std::string> tags;
+
+  for (auto& arg : _args)
+  {
+    if (arg.hasTag ("TAG"))
+      tags.push_back (arg.attribute ("raw"));
+  }
+
+  return tags;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+std::string CLI::getAnnotation () const
+{
+  std::string annotation;
+
+  for (auto& arg : _args)
+  {
+    if (arg.hasTag ("TAG"))
+    {
+      annotation = (arg.attribute ("raw"));
+    }
+
+  }
+
+  return annotation;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+Duration CLI::getDuration () const
+{
+  std::string delta;
+  for (auto& arg : _args)
+  {
+    if (arg.hasTag ("FILTER") &&
+        arg._lextype == Lexer::Type::duration)
+    {
+      delta = arg.attribute ("raw");
+    }
+  }
+  Duration dur (delta);
+  return dur;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+
+std::vector <std::string> CLI::getDomReferences () const
+{
+  std::vector<std::string> references;
+
+  for (auto &arg : _args)
+  {
+    if (arg.hasTag ("DOM"))
+    {
+      references.emplace_back (arg.attribute ("raw"));
+    }
+  }
+
+  return references;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// A filter is just another interval, containing start, end and tags.
+//
+// Supported interval forms:
+//   ["from"] <date> ["to"|"-" <date>]
+//   ["from"] <date> "for" <duration>
+//   <duration> ["before"|"after" <date>]
+//   <duration> "ago"
+//
+Interval CLI::getFilter (const Range& default_range) const
+{
+  // One instance, so we can directly compare.
+  Datetime now;
+
+  Interval filter;
+  std::string start;
+  std::string end;
+  std::string duration;
+  std::vector <std::string> args;
+
+  for (auto& arg : _args)
+  {
+    if (arg.hasTag ("BINARY") ||
+        arg.hasTag ("CMD")    ||
+        arg.hasTag ("EXT"))
+      continue;
+
+    if (arg.hasTag ("FILTER"))
+    {
+      auto canonical = arg.attribute ("canonical");
+      auto raw       = arg.attribute ("raw");
+
+      if (arg.hasTag ("HINT"))
+      {
+        Range range;
+        if (expandIntervalHint (canonical, range))
+        {
+          if (range.is_empty ())
+          {
+            args.push_back ("<all>");
+          }
+          else
+          {
+            start = range.start.toISO ();
+            end   = range.end.toISO ();
+
+            args.push_back ("<date>");
+            args.push_back ("-");
+            args.push_back ("<date>");
+          }
+        }
+
+        // Hints that are not expandable to a date range are ignored.
+      }
+      else if (arg._lextype == Lexer::Type::date)
+      {
+        if (start.empty ())
+          start = raw;
+        else if (end.empty ())
+          end = raw;
+
+        args.push_back ("<date>");
+      }
+      else if (arg._lextype == Lexer::Type::duration)
+      {
+        if (duration.empty ())
+          duration = raw;
+
+        args.push_back ("<duration>");
+      }
+      else if (arg.hasTag ("KEYWORD"))
+      {
+        // Note: that KEYWORDS are not entities (why not?) and there is a list
+        //       in CLI.cpp of them that must be maintained and synced with this
+        //       function.
+        args.push_back (raw);
+      }
+      else if (arg.hasTag ("ID"))
+      {
+        // Not part of a filter.
+      }
+      else
+      {
+        filter.tag (raw);
+      }
+    }
+  }
+
+  if (args.empty ())
+  {
+    filter.setRange(default_range);
+  }
+
+  // <date>
+  else if (args.size () == 1 &&
+      args[0] == "<date>")
+  {
+    DatetimeParser dtp;
+    Range range = dtp.parse_range(start);
+    filter.setRange (range);
+  }
+
+    // from <date>
+  else if (args.size () == 2 &&
+           args[0] == "from" &&
+           args[1] == "<date>")
+  {
+    filter.setRange ({Datetime (start), 0});
+  }
+    // <date> to/- <date>
+  else if (args.size () == 3                   &&
+           args[0] == "<date>"                 &&
+           (args[1] == "to" || args[1] == "-") &&
+           args[2] == "<date>")
+  {
+    filter.setRange ({Datetime (start), Datetime (end)});
+  }
+
+    // from <date> to/- <date>
+  else if (args.size () == 4                   &&
+           args[0] == "from"                   &&
+           args[1] == "<date>"                 &&
+           (args[2] == "to" || args[2] == "-") &&
+           args[3] == "<date>")
+  {
+    filter.setRange ({Datetime (start), Datetime (end)});
+  }
+
+    // <date> for <duration>
+  else if (args.size () == 3   &&
+           args[0] == "<date>" &&
+           args[1] == "for"    &&
+           args[2] == "<duration>")
+  {
+    filter.setRange ({Datetime (start), Datetime (start) + Duration (duration).toTime_t ()});
+  }
+
+    // from <date> for <duration>
+  else if (args.size () == 4       &&
+           args[0] == "from"       &&
+           args[1] == "<date>"     &&
+           args[2] == "for"        &&
+           args[3] == "<duration>")
+  {
+    filter.setRange ({Datetime (start), Datetime (start) + Duration (duration).toTime_t ()});
+  }
+
+    // <duration> before <date>
+  else if (args.size () == 3       &&
+           args[0] == "<duration>" &&
+           args[1] == "before"     &&
+           args[2] == "<date>")
+  {
+    filter.setRange ({Datetime (start) - Duration (duration).toTime_t (), Datetime (start)});
+  }
+
+    // <duration> after <date>
+  else if (args.size () == 3       &&
+           args[0] == "<duration>" &&
+           args[1] == "after"      &&
+           args[2] == "<date>")
+  {
+    filter.setRange ({Datetime (start), Datetime (start) + Duration (duration).toTime_t ()});
+  }
+
+    // <duration> ago
+  else if (args.size () == 2       &&
+           args[0] == "<duration>" &&
+           args[1] == "ago")
+  {
+    filter.setRange ({now - Duration (duration).toTime_t (), 0});
+  }
+
+    // for <duration>
+  else if (args.size () == 2       &&
+           args[0] == "for"        &&
+           args[1] == "<duration>")
+  {
+    filter.setRange ({now - Duration (duration).toTime_t (), now});
+  }
+
+    // <duration>
+  else if (args.size () == 1 &&
+           args[0] == "<duration>")
+  {
+    filter.setRange ({now - Duration (duration).toTime_t (), now});
+  }
+
+  // :all
+  else if (args.size () == 1 && args[0] == "<all>")
+  {
+    filter.setRange (0, 0);
+  }
+
+    // Unrecognized date range construct.
+  else if (! args.empty ())
+  {
+    throw std::string ("Unrecognized date range: '") + join (" ", args) + "'.";
+  }
+
+  if (filter.end != 0 && filter.start > filter.end)
+    throw std::string ("The end of a date range must be after the start.");
+
+  return filter;
+}
