@@ -45,13 +45,11 @@ int CmdChartDay (
   Rules& rules,
   Database& database)
 {
-  // Create a filter, and if empty, choose the current day.
-  auto filter = getFilter (cli);
+  Range default_range = {};
+  expandIntervalHint (rules.get ("reports.day.range", ":day"), default_range);
 
-  if (! filter.is_started ())
-  {
-    expandIntervalHint (rules.get ("reports.day.range", ":day"), filter);
-  }
+  // Create a filter, and if empty, choose the current day.
+  auto filter = cli.getFilter (default_range);
 
   return renderChart (cli, "day", filter, rules, database);
 }
@@ -62,13 +60,11 @@ int CmdChartWeek (
   Rules& rules,
   Database& database)
 {
-  // Create a filter, and if empty, choose the current week.
-  auto filter = getFilter (cli);
+  Range default_range = {};
+  expandIntervalHint (rules.get ("reports.week.range", ":week"), default_range);
 
-  if (! filter.is_started ())
-  {
-    expandIntervalHint (rules.get ("reports.week.range", ":week"), filter);
-  }
+  // Create a filter, and if empty, choose the current week.
+  auto filter = cli.getFilter (default_range);
 
   return renderChart (cli, "week", filter, rules, database);
 }
@@ -79,13 +75,11 @@ int CmdChartMonth (
   Rules& rules,
   Database& database)
 {
-  // Create a filter, and if empty, choose the current month.
-  auto filter = getFilter (cli);
+  Range default_range = {};
+  expandIntervalHint (rules.get ("reports.month.range", ":month"), default_range);
 
-  if (! filter.is_started ())
-  {
-    expandIntervalHint (rules.get ("reports.month.range", ":month"), filter);
-  }
+  // Create a filter, and if empty, choose the current month.
+  auto filter = cli.getFilter (default_range);
 
   return renderChart (cli, "month", filter, rules, database);
 }
@@ -98,7 +92,7 @@ int renderChart (
   Rules& rules,
   Database& database)
 {
-  auto verbose = rules.getBoolean ("verbose");
+  const bool verbose = rules.getBoolean ("verbose");
 
   // Load the data.
   const auto tracked = getTracked (database, rules, filter);
