@@ -38,10 +38,10 @@ int CmdTags (
   Rules& rules,
   Database& database)
 {
-  auto verbose = rules.getBoolean ("verbose");
+  const bool verbose = rules.getBoolean ("verbose");
 
   // Create a filter, with no default range.
-  auto filter = getFilter (cli);
+  auto filter = cli.getFilter ();
 
   // Generate a unique, ordered list of tags.
   std::set <std::string> tags;
