@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2015 - 2016, Paul Beckingham, Federico Hernandez.
+// Copyright 2016 - 2020, Thomas Lauf, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -20,10 +20,13 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
+#include <algorithm>
+#include <deque>
+
 #include <cmake.h>
 #include <shared.h>
 #include <format.h>
@@ -32,195 +35,7 @@
 #include <timew.h>
 #include <algorithm>
 #include <iostream>
-
-////////////////////////////////////////////////////////////////////////////////
-// A filter is just another interval, containing start, end and tags.
-//
-// Supported interval forms:
-//   ["from"] <date> ["to"|"-" <date>]
-//   ["from"] <date> "for" <duration>
-//   <duration> ["before"|"after" <date>]
-//   <duration> "ago"
-//
-Interval getFilter (const CLI& cli)
-{
-  // One instance, so we can directly compare.
-  Datetime now;
-
-  Interval filter;
-  std::string start;
-  std::string end;
-  std::string duration;
-  std::vector <std::string> args;
-  for (auto& arg : cli._args)
-  {
-    if (arg.hasTag ("BINARY") ||
-        arg.hasTag ("CMD")    ||
-        arg.hasTag ("EXT"))
-      continue;
-
-    if (arg.hasTag ("FILTER"))
-    {
-      auto canonical = arg.attribute ("canonical");
-      auto raw       = arg.attribute ("raw");
-
-      if (arg.hasTag ("HINT"))
-      {
-        Range range;
-        if (expandIntervalHint (canonical, range))
-        {
-          start = range.start.toISO ();
-          end   = range.end.toISO ();
-
-          args.push_back ("<date>");
-          args.push_back ("-");
-          args.push_back ("<date>");
-        }
-
-        // Hints that are not expandable to a date range are ignored.
-      }
-      else if (arg._lextype == Lexer::Type::date)
-      {
-        if (start == "")
-          start = raw;
-        else if (end == "")
-          end = raw;
-
-        args.push_back ("<date>");
-      }
-      else if (arg._lextype == Lexer::Type::duration)
-      {
-        if (duration == "")
-          duration = raw;
-
-        args.push_back ("<duration>");
-      }
-      else if (arg.hasTag ("KEYWORD"))
-      {
-        // Note: that KEYWORDS are not entities (why not?) and there is a list
-        //       in CLI.cpp of them that must be maintained and synced with this
-        //       function.
-        args.push_back (raw);
-      }
-      else if (arg.hasTag ("ID"))
-      {
-        // Not part of a filter.
-      }
-      else
-      {
-        filter.tag (raw);
-      }
-    }
-  }
-
-  // <date>
-  if (args.size () == 1 &&
-      args[0] == "<date>")
-  {
-    filter.range = {Datetime (start), 0};
-  }
-
-  // from <date>
-  else if (args.size () == 2 &&
-           args[0] == "from" &&
-           args[1] == "<date>")
-  {
-    filter.range = {Datetime (start), 0};
-  }
-
-  // <date> to/- <date>
-  else if (args.size () == 3                   &&
-           args[0] == "<date>"                 &&
-           (args[1] == "to" || args[1] == "-") &&
-           args[2] == "<date>")
-  {
-    filter.range = {Datetime (start), Datetime (end)};
-  }
-
-  // from <date> to/- <date>
-  else if (args.size () == 4                   &&
-           args[0] == "from"                   &&
-           args[1] == "<date>"                 &&
-           (args[2] == "to" || args[2] == "-") &&
-           args[3] == "<date>")
-  {
-    filter.range = {Datetime (start), Datetime (end)};
-  }
-
-  // <date> for <duration>
-  else if (args.size () == 3   &&
-           args[0] == "<date>" &&
-           args[1] == "for"    &&
-           args[2] == "<duration>")
-  {
-    filter.range = {Datetime (start), Datetime (start) + Duration (duration).toTime_t ()};
-  }
-
-  // from <date> for <duration>
-  else if (args.size () == 4       &&
-           args[0] == "from"       &&
-           args[1] == "<date>"     &&
-           args[2] == "for"        &&
-           args[3] == "<duration>")
-  {
-    filter.range = {Datetime (start), Datetime (start) + Duration (duration).toTime_t ()};
-  }
-
-  // <duration> before <date>
-  else if (args.size () == 3       &&
-           args[0] == "<duration>" &&
-           args[1] == "before"     &&
-           args[2] == "<date>")
-  {
-    filter.range = {Datetime (start) - Duration (duration).toTime_t (), Datetime (start)};
-  }
-
-  // <duration> after <date>
-  else if (args.size () == 3       &&
-           args[0] == "<duration>" &&
-           args[1] == "after"      &&
-           args[2] == "<date>")
-  {
-    filter.range = {Datetime (start), Datetime (start) + Duration (duration).toTime_t ()};
-  }
-
-  // <duration> ago
-  else if (args.size () == 2       &&
-           args[0] == "<duration>" &&
-           args[1] == "ago")
-  {
-    filter.range = {now - Duration (duration).toTime_t (), 0};
-  }
-
-  // for <duration>
-  else if (args.size () == 2       &&
-           args[0] == "for"        &&
-           args[1] == "<duration>")
-  {
-    filter.range = {now - Duration (duration).toTime_t (), now};
-  }
-
-  // <duration>
-  else if (args.size () == 1 &&
-           args[0] == "<duration>")
-  {
-    filter.range = {now - Duration (duration).toTime_t (), now};
-  }
-
-  // Unrecognized date range construct.
-  else if (args.size ())
-  {
-    throw std::string ("Unrecognized date range: '") + join (" ", args) + "'.";
-  }
-
-  if (filter.range.start > now)
-    throw std::string ("Time tracking cannot be set in the future.");
-
-  if (filter.range.end != 0 && filter.range.start > filter.range.end)
-    throw std::string ("The end of a date range must be after the start.");
-
-  return filter;
-}
+#include <IntervalFactory.h>
 
 ////////////////////////////////////////////////////////////////////////////////
 // Read rules and extract all holiday definitions. Create a Range for each
@@ -291,9 +106,9 @@ std::vector <Range> getAllExclusions (
 
   // daysOff are combined with existing holidays.
   results = addRanges (range, results, daysOff);
-  if (daysOn.size ())
+  if (! daysOn.empty ())
     debug (format ("Found {1} additional working days", daysOn.size ()));
-  if (daysOff.size ())
+  if (! daysOff.empty ())
     debug (format ("Found {1} additional non-working days", daysOff.size ()));
 
   // daysOn are subtracted from the existing holidays.
@@ -311,14 +126,127 @@ std::vector <Range> getAllExclusions (
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-std::vector <Interval> getAllInclusions (Database& database)
+void flattenDatabase (Database& database, const Rules& rules)
+{
+  auto latest = getLatestInterval (database);
+  if (latest.is_open ())
+  {
+    auto exclusions = getAllExclusions (rules, {latest.start, Datetime ()});
+    if (! exclusions.empty ())
+    {
+      Interval modified {latest};
+
+      // Update database.
+      database.deleteInterval (latest);
+      for (auto& interval : flatten (modified, exclusions))
+      {
+        database.addInterval (interval, rules.getBoolean ("verbose"));
+      }
+    }
+  }
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// This function will return synthetic intervals
+std::vector <Interval> getIntervalsByIds (
+  Database& database,
+  const Rules& rules,
+  const std::set <int>& ids)
+{
+  std::vector <Interval> intervals;
+  std::deque <Interval> synthetic;
+
+  int current_id = 0;
+  auto id_it = ids.begin ();
+  auto id_end = ids.end ();
+
+  auto it = database.begin ();
+  auto end = database.end ();
+
+  Interval latest;
+  if (it != end )
+  {
+    latest = IntervalFactory::fromSerialization (*it);
+  }
+
+  // If the latest interval is open, check for synthetic intervals
+  if (latest.is_open ())
+  {
+    auto exclusions = getAllExclusions (rules, {latest.start, Datetime ()});
+    if (! exclusions.empty ())
+    {
+        // We're converting the latest interval into synthetic intervals so we need to skip it
+        ++it;
+
+        for (auto& interval : flatten (latest, exclusions))
+        {
+          ++current_id;
+          interval.synthetic = true;
+          interval.id = current_id;
+          synthetic.push_front (interval);
+        }
+    }
+  }
+
+  intervals.reserve (ids.size ());
+
+  // First look for the ids in our set of synthetic intervals
+  for (auto& interval : synthetic)
+  {
+    if (id_it == id_end)
+    {
+      break;
+    }
+
+    if (interval.id == *id_it)
+    {
+      intervals.push_back (interval);
+      ++id_it;
+    }
+  }
+
+  current_id = synthetic.size ();
+
+  // We'll find remaining intervals from the database itself
+  for ( ; it != end; ++it)
+  {
+    ++current_id;
+
+    if (id_it == id_end)
+    {
+      break;
+    }
+
+    if (current_id == *id_it)
+    {
+      Interval interval = IntervalFactory::fromSerialization (*it);
+      interval.id = current_id;
+      intervals.push_back (interval);
+      ++id_it;
+    }
+  }
+
+  // We did not find all the ids we were looking for.
+  if (id_it != id_end)
+  {
+    throw format("ID '@{1}' does not correspond to any tracking.", *id_it);
+  }
+
+  return intervals;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+std::vector <Interval> subset (
+  const Interval& filter,
+  const std::deque <Interval>& intervals)
 {
   std::vector <Interval> all;
-  for (auto& line : database.allLines ())
+  for (auto& interval : intervals)
   {
-    Interval i;
-    i.initialize (line);
-    all.push_back (i);
+    if (matchesFilter (interval, filter))
+    {
+      all.push_back (interval);
+    }
   }
 
   return all;
@@ -343,9 +271,11 @@ std::vector <Range> subset (
   const std::vector <Range>& ranges)
 {
   std::vector <Range> all;
-  for (auto& r : ranges)
-    if (range.overlap (r))
+  for (auto& r : ranges) {
+    if (range.intersects (r)) {
       all.push_back (r);
+    }
+  }
 
   return all;
 }
@@ -356,9 +286,12 @@ std::vector <Interval> subset (
   const std::vector <Interval>& intervals)
 {
   std::vector <Interval> all;
-  for (auto& interval : intervals)
-    if (range.overlap (interval.range))
+  for (auto& interval : intervals) {
+    if (range.intersects (interval))
+    {
       all.push_back (interval);
+    }
+  }
 
   return all;
 }
@@ -372,33 +305,21 @@ std::vector <Interval> flatten (
 
   std::vector <Range> enclosed;
   for (auto& e : exclusions)
-    if (interval.range.encloses (e))
+    if (interval.encloses (e))
       enclosed.push_back (e);
 
   Datetime now;
-  for (auto& result : subtractRanges ({interval.range}, enclosed))
+  for (auto& result : subtractRanges ({interval}, enclosed))
   {
-    Interval chunk {interval};
-    chunk.range = result;
-
-    // Only historical data is included.
-    if (chunk.range.start <= now)
+    if (interval.is_open() && result.start > now)
     {
-      // Closed chunk ranges in the future need to be adjusted.
-      if (! chunk.range.is_open ()  &&
-          chunk.range.end > now)
-      {
-        // If the interval is open, so must be chunk.
-        if (interval.range.is_open ())
-          chunk.range.end = {0};
-
-        // Otherwise truncate to now.
-        else
-          chunk.range.end = now;
-      }
-
-      all.push_back (chunk);
+      break;
     }
+
+    Interval chunk {interval};
+    chunk.setRange (result);
+
+    all.push_back (chunk);
   }
 
   return all;
@@ -425,7 +346,7 @@ std::vector <Range> merge (
   int merges = 0;
   for (unsigned int i = 0; i < sorted.size (); ++i)
   {
-    if (cursor && sorted[cursor - 1].overlap (sorted[i]))
+    if (cursor && sorted[cursor - 1].overlaps (sorted[i]))
     {
       sorted[cursor - 1] = sorted[cursor - 1].combine (sorted[i]);
       ++merges;
@@ -453,19 +374,18 @@ std::vector <Range> addRanges (
   std::vector <Range> results;
 
   for (auto& range : ranges)
-    if (limits.overlap (range))
+    if (limits.overlaps (range))
       results.push_back (range);
 
   for (auto& addition : additions)
-    if (limits.overlap (addition))
+    if (limits.overlaps (addition))
       results.push_back (addition);
 
   return results;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// Subtract a set of Ranges from another set of Ranges, all within a defined
-// range.
+// Subtract a set of Ranges from another set of Ranges, all within a defined range.
 std::vector <Range> subtractRanges (
   const std::vector <Range>& ranges,
   const std::vector <Range>& subtractions)
@@ -485,38 +405,20 @@ std::vector <Range> subtractRanges (
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// From a set of intervals, find the earliest start and the latest end, and
-// return these in a Range.
-Range outerRange (const std::vector <Interval>& intervals)
+// An interval matches a filter range if the start/end overlaps
+bool matchesRange (const Interval& interval, const Range& filter)
 {
-  Range outer;
-  for (auto& interval : intervals)
-  {
-    if (interval.range.start < outer.start || outer.start.toEpoch () == 0)
-      outer.start = interval.range.start;
-
-    // Deliberately mixed start/end.
-    if (interval.range.start > outer.end)
-      outer.end = interval.range.start;
-
-    if (interval.range.end > outer.end)
-      outer.end = interval.range.end;
-
-    if (! interval.range.is_ended ())
-      outer.end = Datetime ();
-  }
-
-  return outer;
+  return ((!filter.is_started() && !filter.is_ended()) || interval.intersects (filter));
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // An interval matches a filter interval if the start/end overlaps, and all
 // filter interval tags are found in the interval.
 //
-// [1] interval.range.end.toEpoch () == 0
-// [2] interval.range.end > filter.range.start
-// [3] filter.range.end.toEpoch () == 0
-// [4] interval.range.start < filter.range.end
+// [1] interval.end.toEpoch () == 0
+// [2] interval.end > filter.start
+// [3] filter.end.toEpoch () == 0
+// [4] interval.start < filter.end
 //
 // Match:   (1 || 2) && (3 || 4)
 
@@ -548,18 +450,15 @@ Range outerRange (const std::vector <Interval>& intervals)
 //
 bool matchesFilter (const Interval& interval, const Interval& filter)
 {
-  if ((filter.range.start.toEpoch () == 0 &&
-       filter.range.end.toEpoch () == 0)
-
-      ||
-
-      ((interval.range.end.toEpoch () == 0 || interval.range.end   > filter.range.start) &&
-       (filter.range.end.toEpoch ()   == 0 || interval.range.start < filter.range.end)))
+  if (matchesRange (interval, filter))
   {
     for (auto& tag : filter.tags ())
+    {
       if (! interval.hasTag (tag))
+      {
         return false;
-
+      }
+    }
     return true;
   }
 
@@ -567,7 +466,7 @@ bool matchesFilter (const Interval& interval, const Interval& filter)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// Take an interval and clip it to the range.
+// Take an interval and clip it to the range
 Interval clip (const Interval& interval, const Range& range)
 {
   if (! range.is_started () ||
@@ -575,13 +474,13 @@ Interval clip (const Interval& interval, const Range& range)
     return interval;
 
   Interval clipped {interval};
-  if (clipped.range.start.toEpoch () &&
-      clipped.range.start < range.start)
-    clipped.range.start = range.start;
+  if (clipped.start.toEpoch () &&
+      clipped.start < range.start)
+    clipped.start = range.start;
 
-  if (clipped.range.end.toEpoch () &&
-      clipped.range.end > range.end)
-    clipped.range.end = range.end;
+  if (clipped.end.toEpoch () &&
+      clipped.end > range.end)
+    clipped.end = range.end;
 
   return clipped;
 }
@@ -592,39 +491,41 @@ std::vector <Interval> getTracked (
   const Rules& rules,
   Interval& filter)
 {
-  auto inclusions = getAllInclusions (database);
+  int id_skip = 0;
+  std::deque <Interval> intervals;
 
-  // Exclusions are only usable within a range, so if no filter range exists,
-  // determine the outermost range of the inclusions, ie:
-  //
-  //   [earliest start, latest end)
-  //
-  // Avoid assigning a zero-width range - leave it unstarted instead.
-  if (! filter.range.is_started ())
+  for (auto& line : database)
   {
-    auto outer = outerRange (inclusions);
-    if (outer.total ())
-      filter.range = outer;
+    Interval interval = IntervalFactory::fromSerialization(line);
 
-    if (inclusions.size() > 0) {
-      auto latest = inclusions.back();
-      if (latest.range.is_open()) {;
-        filter.range.end = 0;
-      }
+    // Since we are moving backwards, and the intervals are in sorted order,
+    // if the filter is after the interval, we know there will be no more matches
+    if (matchesRange (interval, filter))
+    {
+      intervals.push_front (std::move (interval));
+    }
+    else if (interval.start.toEpoch () >= filter.start.toEpoch ())
+    {
+      ++id_skip;
+    }
+    else
+    {
+      break;
     }
   }
 
-  std::vector <Interval> intervals = inclusions;
-
-  if (intervals.size () > 0)
+  if (! intervals.empty ())
   {
-    auto latest = inclusions.back ();
+    auto latest = intervals.back ();
 
-    if (latest.range.is_open ())
+    if (latest.is_open ())
     {
-      // Get the set of expanded exclusions that overlap the range defined by the
-      // timeline.
-      auto exclusions = getAllExclusions (rules, filter.range);
+      // Get the set of expanded exclusions that overlap the range defined by the open interval.
+      Interval exclusion_range {};
+      exclusion_range.start = latest.start;
+      exclusion_range.end = Datetime();
+
+      auto exclusions = getAllExclusions (rules, exclusion_range);
       if (! exclusions.empty ())
       {
         intervals.pop_back ();
@@ -632,7 +533,7 @@ std::vector <Interval> getTracked (
         for (auto& interval : flatten (latest, exclusions))
         {
           if (latest.synthetic ||
-              latest.range != interval.range)
+              latest != interval)
             interval.synthetic = true;
 
           intervals.push_back (interval);
@@ -641,12 +542,35 @@ std::vector <Interval> getTracked (
     }
   }
 
-  // Assign an ID to each interval.
+  // Assign an ID to each interval before we prune ones that do not have matching tags
   for (unsigned int i = 0; i < intervals.size (); ++i)
-    intervals[i].id = intervals.size () - i;
+  {
+    intervals[i].id = intervals.size () - i + id_skip;
+  }
+
+  // Now prune the intervals without matching tags
+  if (filter.tags ().size () > 0)
+  {
+    auto cmp = [&filter] (const Interval& interval)
+               {
+                 for (const auto& tag : filter.tags ())
+                 {
+                   if (! interval.hasTag (tag))
+                   {
+                     return true;
+                   }
+                 }
+                 return false;
+               };
+
+    intervals.erase (std::remove_if (intervals.begin (), intervals.end (), cmp),
+                     intervals.end ());
+  }
 
   debug (format ("Loaded {1} tracked intervals", intervals.size ()));
-  return subset (filter, intervals);
+
+  return std::vector <Interval> (std::make_move_iterator (intervals.begin ()),
+                                 std::make_move_iterator (intervals.end ()));
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -656,11 +580,26 @@ std::vector <Range> getUntracked (
   const Rules& rules,
   Interval& filter)
 {
+  bool found_match = false;
   std::vector <Range> inclusion_ranges;
-  for (auto& i : subset (filter, getAllInclusions (database)))
-    inclusion_ranges.push_back (i.range);
+  for (auto& line : database)
+  {
+    Interval i = IntervalFactory::fromSerialization (line);
+    if (matchesFilter (i, filter))
+    {
+      inclusion_ranges.push_back (i);
+      found_match = true;
+    }
+    else if (found_match)
+    {
+      // If we already had a match, and now we do not, since the database is in
+      // order from most recent to oldest inclusion, we can be sure that there
+      // will not be any further matches.
+      break;
+    }
+  }
 
-  auto available = subtractRanges ({filter.range}, getAllExclusions (rules, filter.range));
+  auto available = subtractRanges ({filter}, getAllExclusions (rules, filter));
   auto untracked = subtractRanges (available, inclusion_ranges);
   debug (format ("Loaded {1} untracked ranges", untracked.size ()));
   return untracked;
@@ -670,21 +609,13 @@ std::vector <Range> getUntracked (
 Interval getLatestInterval (Database& database)
 {
   Interval i;
-  for (auto& line : database.allLines ())
+  auto latestEntry = database.getLatestEntry ();
+  if (! latestEntry.empty ())
   {
-    // inc YYYYMMDDTHHMMSSZ - YYYYMMDDTHHMMSSZ # ...
-    //                     ^ 20
-    if (line.find (" - ") != 20)
-    {
-      i.initialize (line);
-      return i;
-    }
+    i = IntervalFactory::fromSerialization (latestEntry);
+    i.id = 1;
   }
 
-  auto lastLine = database.lastLine ();
-  if (lastLine != "")
-    i.initialize (lastLine);
-
   return i;
 }
 
